/**
 * JBoss, Home of Professional Open Source
 * Copyright 2011, Red Hat, Inc. and/or its affiliates, and individual
 * contributors by the @authors tag. See the copyright.txt in the
 * distribution for a full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.seam.reports.xdocreport.test;

import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.enterprise.inject.Default;
import javax.inject.Inject;

import org.apache.tika.Tika;
import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.seam.reports.Report;
import org.jboss.seam.reports.ReportDefinition;
import org.jboss.seam.reports.ReportLoader;
import org.jboss.seam.reports.ReportRenderer;
import org.jboss.seam.reports.output.HTML;
import org.jboss.seam.reports.output.PDF;
import org.jboss.seam.reports.test.Utils;
import org.jboss.seam.reports.xdocreport.annotations.XDocReport;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.jboss.solder.resourceLoader.Resource;
import org.junit.Test;
import org.junit.runner.RunWith;

import de.oio.jpdfunit.DocumentTester;
import de.oio.jpdfunit.document.util.TextSearchType;

@RunWith(Arquillian.class)
public class XDocReportsTest {

    @Inject
    @XDocReport
    @Default
    ReportRenderer reportRenderer;

    @Inject
    @XDocReport
    ReportLoader reportLoader;

    @Deployment(name = "XDocReports")
    public static WebArchive createArchive() {
        return Utils.getDeploymentFactory().xdocreportDeployment().addClasses(XDocReportsTest.class)
                .addAsResource("DocxProjectWithVelocity.docx", "DocxProjectWithVelocity.docx")
                .addAsResource("ODTProjectWithVelocity.odt", "ODTProjectWithVelocity.odt");
    }

    @Test
    public void testDocxWithMap(@Resource("DocxProjectWithVelocity.docx") InputStream sourceReport) throws Exception {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        ReportDefinition reportDefinition = reportLoader.loadReportDefinition(sourceReport);
        Map<String, String> dataSource = Collections.singletonMap("project", "Seam Reports");
        Report report = reportDefinition.fill(dataSource, null);
        reportRenderer.render(report, output);
        // Extracting the result
        String text = new Tika().parseToString(new ByteArrayInputStream(output.toByteArray()));
        assertTrue(text.contains("Seam Reports Rocks"));
    }

    @Test
    public void testDocxWithMapAsParameter(@Resource("DocxProjectWithVelocity.docx") InputStream sourceReport) throws Exception {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        ReportDefinition reportDefinition = reportLoader.loadReportDefinition(sourceReport);
        Map<String, Object> parameters = Collections.singletonMap("project", (Object) "Seam Reports");
        Report report = reportDefinition.fill(null, parameters);
        reportRenderer.render(report, output);
        // Extracting the result
        String text = new Tika().parseToString(new ByteArrayInputStream(output.toByteArray()));
        assertTrue(text.contains("Seam Reports Rocks"));
    }

    @Test
    public void testOdtxWithMap(@Resource("ODTProjectWithVelocity.odt") InputStream sourceReport) throws Exception {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        ReportDefinition reportDefinition = reportLoader.loadReportDefinition(sourceReport);
        Map<String, String> dataSource = Collections.singletonMap("project", "Seam Reports");
        Report report = reportDefinition.fill(dataSource, null);
        reportRenderer.render(report, output);
        // Extracting the result
        String text = new Tika().parseToString(new ByteArrayInputStream(output.toByteArray()));
        assertTrue(text.contains("Seam Reports Rocks"));
    }

    @Test
    public void testOdtxWithMapAsParameter(@Resource("ODTProjectWithVelocity.odt") InputStream sourceReport) throws Exception {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        ReportDefinition reportDefinition = reportLoader.loadReportDefinition(sourceReport);
        Map<String, Object> parameters = Collections.singletonMap("project", (Object) "Seam Reports");
        Report report = reportDefinition.fill(null, parameters);
        reportRenderer.render(report, output);
        // Extracting the result
        String text = new Tika().parseToString(new ByteArrayInputStream(output.toByteArray()));
        assertTrue(text.contains("Seam Reports Rocks"));
    }

    @Test
    public void testConversionFromDocxToPDF(@Resource("DocxProjectWithVelocity.docx") InputStream sourceReport,
            @XDocReport @PDF ReportRenderer pdfReportRenderer) throws Exception {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        ReportDefinition reportDefinition = reportLoader.loadReportDefinition(sourceReport);
        Map<String, Object> dataSource = new HashMap<String, Object>();
        dataSource.put("project", "Seam Reports");
        Report report = reportDefinition.fill(dataSource, null);
        pdfReportRenderer.render(report, output);
        // Testing the generated PDF
        DocumentTester tester = new DocumentTester(new ByteArrayInputStream(output.toByteArray()));
        tester.assertContentContainsText("Seam Reports Rocks", TextSearchType.CONTAINS);
    }

    @Test
    public void testConversionFromDocxToHTML(@Resource("DocxProjectWithVelocity.docx") InputStream sourceReport,
            @XDocReport @HTML ReportRenderer htmlReportRenderer) throws Exception {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        ReportDefinition reportDefinition = reportLoader.loadReportDefinition(sourceReport);
        Map<String, Object> dataSource = new HashMap<String, Object>();
        dataSource.put("project", "Seam Reports");
        Report report = reportDefinition.fill(dataSource, null);
        htmlReportRenderer.render(report, output);
        // Testing the generated HTML
        assertTrue(output.toString().contains("<span style=\"font-size:-1.0pt;\">Seam Reports</span><span style=\"font-size:-1.0pt;\"> Rocks&nbsp;!</span>"));
    }
}